@@ -135,27 +135,96 @@ tp_get_touch(struct tp_dispatch *tp, unsigned int slot)
 	return &tp->touches[slot];
 }
 
-static inline void
-tp_begin_touch(struct tp_dispatch *tp, struct tp_touch *t, uint64_t time)
+static inline unsigned int
+tp_fake_finger_count(struct tp_dispatch *tp)
 {
-	if (t->state == TOUCH_BEGIN || t->state == TOUCH_UPDATE)
+	/* don't count BTN_TOUCH */
+	return ffs(tp->fake_touches >> 1);
+}
+
+static inline bool
+tp_fake_finger_is_touching(struct tp_dispatch *tp)
+{
+	return tp->fake_touches & 0x1;
+}
+
+static inline void
+tp_fake_finger_set(struct tp_dispatch *tp,
+		   unsigned int code,
+		   bool is_press)
+{
+	unsigned int shift;
+
+	switch (code) {
+	case BTN_TOUCH:
+		shift = 0;
+		break;
+	case BTN_TOOL_FINGER:
+		shift = 1;
+		break;
+	case BTN_TOOL_DOUBLETAP:
+	case BTN_TOOL_TRIPLETAP:
+	case BTN_TOOL_QUADTAP:
+		shift = code - BTN_TOOL_DOUBLETAP + 2;
+		break;
+	default:
+		return;
+	}
+
+	if (is_press)
+		tp->fake_touches |= 1 << shift;
+	else
+		tp->fake_touches &= ~(0x1 << shift);
+}
+
+static inline void
+tp_new_touch(struct tp_dispatch *tp, struct tp_touch *t, uint64_t time)
+{
+	if (t->state == TOUCH_BEGIN ||
+	    t->state == TOUCH_UPDATE ||
+	    t->state == TOUCH_HOVERING)
 		return;
 
+	/* we begin the touch as hovering because until BTN_TOUCH happens we
+	 * don't know if it's a touch down or not. And BTN_TOUCH may happen
+	 * after ABS_MT_TRACKING_ID */
 	tp_motion_history_reset(t);
 	t->dirty = true;
-	t->state = TOUCH_BEGIN;
+	t->has_ended = false;
+	t->state = TOUCH_HOVERING;
 	t->pinned.is_pinned = false;
 	t->millis = time;
-	tp->nfingers_down++;
-	assert(tp->nfingers_down >= 1);
 	tp->queued |= TOUCHPAD_EVENT_MOTION;
 }
 
+static inline void
+tp_begin_touch(struct tp_dispatch *tp, struct tp_touch *t, uint64_t time)
+{
+	t->dirty = true;
+	t->state = TOUCH_BEGIN;
+	t->millis = time;
+	tp->nfingers_down++;
+	assert(tp->nfingers_down >= 1);
+}
+
+/**
+ * End a touch, even if the touch sequence is still active.
+ */
 static inline void
 tp_end_touch(struct tp_dispatch *tp, struct tp_touch *t, uint64_t time)
 {
-	if (t->state == TOUCH_END || t->state == TOUCH_NONE)
+	switch (t->state) {
+	case TOUCH_HOVERING:
+		t->state = TOUCH_NONE;
+		/* fallthough */
+	case TOUCH_NONE:
+	case TOUCH_END:
 		return;
+	case TOUCH_BEGIN:
+	case TOUCH_UPDATE:
+		break;
+
+	}
 
 	t->dirty = true;
 	t->is_pointer = false;
@@ -168,6 +237,16 @@ tp_end_touch(struct tp_dispatch *tp, struct tp_touch *t, uint64_t time)
 	tp->queued |= TOUCHPAD_EVENT_MOTION;
 }
 
+/**
+ * End the touch sequence on ABS_MT_TRACKING_ID -1 or when the BTN_TOOL_* 0 is received.
+ */
+static inline void
+tp_end_sequence(struct tp_dispatch *tp, struct tp_touch *t, uint64_t time)
+{
+	t->has_ended = true;
+	tp_end_touch(tp, t, time);
+}
+
 static double
 tp_estimate_delta(int x0, int x1, int x2, int x3)
 {
@@ -218,9 +297,9 @@ tp_process_absolute(struct tp_dispatch *tp,
 		break;
 	case ABS_MT_TRACKING_ID:
 		if (e->value != -1)
-			tp_begin_touch(tp, t, time);
+			tp_new_touch(tp, t, time);
 		else
-			tp_end_touch(tp, t, time);
+			tp_end_sequence(tp, t, time);
 	}
 }
 
@@ -253,41 +332,56 @@ tp_process_fake_touch(struct tp_dispatch *tp,
 		      uint64_t time)
 {
 	struct tp_touch *t;
-	unsigned int fake_touches;
 	unsigned int nfake_touches;
 	unsigned int i, start;
-	unsigned int shift;
 
-	if (e->code != BTN_TOUCH &&
-	    (e->code < BTN_TOOL_DOUBLETAP || e->code > BTN_TOOL_QUADTAP))
-		return;
+	tp_fake_finger_set(tp, e->code, e->value != 0);
 
-	shift = e->code == BTN_TOUCH ? 0 : (e->code - BTN_TOOL_DOUBLETAP + 1);
+	nfake_touches = tp_fake_finger_count(tp);
 
-	if (e->value)
-		tp->fake_touches |= 1 << shift;
-	else
-		tp->fake_touches &= ~(0x1 << shift);
-
-	fake_touches = tp->fake_touches;
-	nfake_touches = 0;
-	while (fake_touches) {
-		nfake_touches++;
-		fake_touches >>= 1;
-	}
-
-	/* For single touch tps we use BTN_TOUCH for begin / end of touch 0 */
 	start = tp->has_mt ? tp->real_touches : 0;
 	for (i = start; i < tp->ntouches; i++) {
 		t = tp_get_touch(tp, i);
 		if (i < nfake_touches)
-			tp_begin_touch(tp, t, time);
+			tp_new_touch(tp, t, time);
 		else
-			tp_end_touch(tp, t, time);
+			tp_end_sequence(tp, t, time);
+	}
+}
+
+static void
+tp_process_trackpoint_button(struct tp_dispatch *tp,
+			     const struct input_event *e,
+			     uint64_t time)
+{
+	struct evdev_dispatch *dispatch;
+	struct input_event event;
+
+	if (!tp->buttons.trackpoint ||
+	    (tp->device->tags & EVDEV_TAG_TOUCHPAD_TRACKPOINT) == 0)
+		return;
+
+	dispatch = tp->buttons.trackpoint->dispatch;
+
+	event = *e;
+
+	switch (event.code) {
+	case BTN_0:
+		event.code = BTN_LEFT;
+		break;
+	case BTN_1:
+		event.code = BTN_RIGHT;
+		break;
+	case BTN_2:
+		event.code = BTN_MIDDLE;
+		break;
+	default:
+		return;
 	}
 
-	/* On mt the actual touch info may arrive after BTN_TOOL_FOO */
-	assert(tp->has_mt || tp->nfingers_down == nfake_touches);
+	dispatch->interface->process(dispatch,
+				     tp->buttons.trackpoint,
+				     &event, time);
 }
 
 static void
@@ -302,11 +396,17 @@ tp_process_key(struct tp_dispatch *tp,
 			tp_process_button(tp, e, time);
 			break;
 		case BTN_TOUCH:
+		case BTN_TOOL_FINGER:
 		case BTN_TOOL_DOUBLETAP:
 		case BTN_TOOL_TRIPLETAP:
 		case BTN_TOOL_QUADTAP:
 			tp_process_fake_touch(tp, e, time);
 			break;
+		case BTN_0:
+		case BTN_1:
+		case BTN_2:
+			tp_process_trackpoint_button(tp, e, time);
+			break;
 	}
 }
 
@@ -325,7 +425,12 @@ tp_unpin_finger(struct tp_dispatch *tp, struct tp_touch *t)
 			tp->buttons.motion_dist * tp->buttons.motion_dist) {
 		t->pinned.is_pinned = false;
 		tp_set_pointer(tp, t);
+		return;
 	}
+
+	/* The finger may slowly drift, adjust the center */
+	t->pinned.center_x = t->x + t->pinned.center_x / 2;
+	t->pinned.center_y = t->y + t->pinned.center_y / 2;
 }
 
 static void
@@ -554,6 +659,61 @@ tp_remove_scroll(struct tp_dispatch *tp)
 	tp_remove_edge_scroll(tp);
 }
 
+static void
+tp_unhover_touches(struct tp_dispatch *tp, uint64_t time)
+{
+	struct tp_touch *t;
+	unsigned int nfake_touches;
+	int i;
+
+	if (!tp->fake_touches && !tp->nfingers_down)
+		return;
+
+	nfake_touches = tp_fake_finger_count(tp);
+	if (tp->nfingers_down == nfake_touches &&
+	    ((tp->nfingers_down == 0 && !tp_fake_finger_is_touching(tp)) ||
+	     (tp->nfingers_down > 0 && tp_fake_finger_is_touching(tp))))
+		return;
+
+	/* if BTN_TOUCH is set and we have less fingers down than fake
+	 * touches, switch each hovering touch to BEGIN
+	 * until nfingers_down matches nfake_touches
+	 */
+	if (tp_fake_finger_is_touching(tp) &&
+	    tp->nfingers_down < nfake_touches) {
+		for (i = 0; i < (int)tp->ntouches; i++) {
+			t = tp_get_touch(tp, i);
+
+			if (t->state == TOUCH_HOVERING) {
+				tp_begin_touch(tp, t, time);
+
+				if (tp->nfingers_down >= nfake_touches)
+					break;
+			}
+		}
+	}
+
+	/* if BTN_TOUCH is unset end all touches, we're hovering now. If we
+	 * have too many touches also end some of them. This is done in
+	 * reverse order.
+	 */
+	if (tp->nfingers_down > nfake_touches ||
+	    !tp_fake_finger_is_touching(tp)) {
+		for (i = tp->ntouches - 1; i >= 0; i--) {
+			t = tp_get_touch(tp, i);
+
+			if (t->state == TOUCH_HOVERING)
+				continue;
+
+			tp_end_touch(tp, t, time);
+
+			if (tp_fake_finger_is_touching(tp) &&
+			    tp->nfingers_down == nfake_touches)
+				break;
+		}
+	}
+}
+
 static void
 tp_process_state(struct tp_dispatch *tp, uint64_t time)
 {
@@ -561,6 +721,8 @@ tp_process_state(struct tp_dispatch *tp, uint64_t time)
 	struct tp_touch *first = tp_get_touch(tp, 0);
 	unsigned int i;
 
+	tp_unhover_touches(tp, time);
+
 	for (i = 0; i < tp->ntouches; i++) {
 		t = tp_get_touch(tp, i);
 
@@ -606,13 +768,18 @@ tp_post_process_state(struct tp_dispatch *tp, uint64_t time)
 	struct tp_touch *t;
 
 	tp_for_each_touch(tp, t) {
+
 		if (!t->dirty)
 			continue;
 
-		if (t->state == TOUCH_END)
-			t->state = TOUCH_NONE;
-		else if (t->state == TOUCH_BEGIN)
+		if (t->state == TOUCH_END) {
+			if (t->has_ended)
+				t->state = TOUCH_NONE;
+			else
+				t->state = TOUCH_HOVERING;
+		} else if (t->state == TOUCH_BEGIN) {
 			t->state = TOUCH_UPDATE;
+		}
 
 		t->dirty = false;
 	}
@@ -793,7 +960,7 @@ tp_clear_state(struct tp_dispatch *tp)
 	tp_release_all_taps(tp, now);
 
 	tp_for_each_touch(tp, t) {
-		tp_end_touch(tp, t, now);
+		tp_end_sequence(tp, t, now);
 	}
 
 	tp_handle_state(tp, now);
@@ -811,7 +978,7 @@ tp_suspend(struct tp_dispatch *tp, struct evdev_device *device)
 	if (tp->buttons.has_topbuttons) {
 		evdev_notify_suspended_device(device);
 		/* Enlarge topbutton area while suspended */
-		tp_init_softbuttons(tp, device, 1.5);
+		tp_init_top_softbuttons(tp, device, 1.5);
 	} else {
 		evdev_device_suspend(device);
 	}
@@ -824,7 +991,7 @@ tp_resume(struct tp_dispatch *tp, struct evdev_device *device)
 		/* tap state-machine is offline while suspended, reset state */
 		tp_clear_state(tp);
 		/* restore original topbutton area size */
-		tp_init_softbuttons(tp, device, 1.0);
+		tp_init_top_softbuttons(tp, device, 1.0);
 		evdev_notify_resumed_device(device);
 	} else {
 		evdev_device_resume(device);
@@ -936,6 +1103,10 @@ tp_tag_device(struct evdev_device *device,
 			 device->tags |= EVDEV_TAG_INTERNAL_TOUCHPAD;
 	} else if (bustype != BUS_BLUETOOTH)
 		device->tags |= EVDEV_TAG_INTERNAL_TOUCHPAD;
+
+	if (udev_device_get_property_value(udev_device,
+					   "TOUCHPAD_HAS_TRACKPOINT_BUTTONS"))
+		device->tags |= EVDEV_TAG_TOUCHPAD_TRACKPOINT;
 }
 
 static struct evdev_dispatch_interface tp_interface = {
@@ -954,6 +1125,7 @@ tp_init_touch(struct tp_dispatch *tp,
 	      struct tp_touch *t)
 {
 	t->tp = tp;
+	t->has_ended = true;
 }
 
 static int
