@@ -1229,10 +1229,12 @@ evdev_device_init_pointer_acceleration(struct evdev_device *device)
 	device->pointer.config.get_default_speed = evdev_accel_config_get_default_speed;
 	device->base.config.accel = &device->pointer.config;
 
+	evdev_accel_config_set_speed(&device->base,
+		     evdev_accel_config_get_default_speed(&device->base));
+
 	return 0;
 }
 
-
 static inline int
 evdev_need_mtdev(struct evdev_device *device)
 {
@@ -1324,7 +1326,7 @@ evdev_configure_device(struct evdev_device *device)
 	struct libevdev *evdev = device->evdev;
 	const struct input_absinfo *absinfo;
 	int has_abs, has_rel, has_mt;
-	int has_button, has_keyboard, has_touch;
+	int has_button, has_keyboard, has_touch, has_joystick_button;
 	struct mt_slot *slots;
 	int num_slots;
 	int active_slot;
@@ -1336,17 +1338,24 @@ evdev_configure_device(struct evdev_device *device)
 	has_abs = 0;
 	has_mt = 0;
 	has_button = 0;
+	has_joystick_button = 0;
 	has_keyboard = 0;
 	has_touch = 0;
 
-        for (i = BTN_JOYSTICK; i < BTN_DIGI; i++) {
-                if (libevdev_has_event_code(evdev, EV_KEY, i)) {
-                        log_info(libinput,
-                                 "input device '%s', %s is a joystick, ignoring\n",
-                                 device->devname, devnode);
-                        return -1;
-                }
-        }
+	for (i = BTN_JOYSTICK; i <= BTN_PINKIE; i++)
+		if (libevdev_has_event_code(evdev, EV_KEY, i))
+			has_joystick_button = 1;
+
+	for (i = BTN_GAMEPAD; i <= BTN_TR2; i++)
+		if (libevdev_has_event_code(evdev, EV_KEY, i))
+			has_joystick_button = 1;
+
+	if (has_joystick_button) {
+		log_info(libinput,
+			 "input device '%s', %s is a joystick, ignoring\n",
+			 device->devname, devnode);
+		return -1;
+	}
 
 	if (libevdev_has_event_type(evdev, EV_ABS)) {
 
@@ -1464,7 +1473,8 @@ evdev_configure_device(struct evdev_device *device)
 		has_keyboard = 1;
 
 	if ((has_abs || has_rel) && has_button) {
-		if (evdev_device_init_pointer_acceleration(device) == -1)
+		if (has_rel &&
+		    evdev_device_init_pointer_acceleration(device) == -1)
 			return -1;
 
 		device->seat_caps |= EVDEV_DEVICE_POINTER;
