@@ -131,6 +131,9 @@ ratelimit_test(struct ratelimit *r)
  * use a reasonable default instead. If the property contains multiple DPI
  * settings but none flagged as default, we return the last because we're
  * lazy and that's a silly way to set the property anyway.
+ *
+ * @param prop The value of the udev property (without the MOUSE_DPI=)
+ * @return The default dpi value on success, 0 on error
  */
 int
 parse_mouse_dpi_property(const char *prop)
@@ -168,3 +171,33 @@ parse_mouse_dpi_property(const char *prop)
 	}
 	return dpi;
 }
+
+/**
+ * Helper function to parse the MOUSE_WHEEL_CLICK_ANGLE property from udev.
+ * Property is of the form:
+ * MOUSE_WHEEL_CLICK_ANGLE=<integer>
+ * Where the number indicates the degrees travelled for each click.
+ *
+ * We skip preceding whitespaces and parse the first number seen. If
+ * multiple numbers are specified, we ignore those.
+ *
+ * @param prop The value of the udev property (without the MOUSE_WHEEL_CLICK_ANGLE=)
+ * @return The angle of the wheel (may be negative) or 0 on error.
+ */
+int
+parse_mouse_wheel_click_angle_property(const char *prop)
+{
+	int angle = 0,
+	    nread = 0;
+
+	while(*prop != 0 && *prop == ' ')
+		prop++;
+
+	sscanf(prop, "%d%n", &angle, &nread);
+	if (nread == 0 || angle == 0 || abs(angle) > 360)
+		return 0;
+	if (prop[nread] != ' ' && prop[nread] != '\0')
+		return 0;
+
+        return angle;
+}
