@@ -26,6 +26,7 @@
 
 #include <assert.h>
 #include <check.h>
+#include <dirent.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <getopt.h>
@@ -44,6 +45,9 @@
 #include "litest-int.h"
 #include "libinput-util.h"
 
+#define UDEV_RULES_D "/run/udev/rules.d"
+#define UDEV_RULE_PREFIX "99-litest-"
+
 static int in_debugger = -1;
 static int verbose = 0;
 
@@ -90,6 +94,8 @@ extern struct litest_test_device litest_generic_singletouch_device;
 extern struct litest_test_device litest_qemu_tablet_device;
 extern struct litest_test_device litest_xen_virtual_pointer_device;
 extern struct litest_test_device litest_vmware_virtmouse_device;
+extern struct litest_test_device litest_synaptics_hover_device;
+extern struct litest_test_device litest_synaptics_carbon3rd_device;
 
 struct litest_test_device* devices[] = {
 	&litest_synaptics_clickpad_device,
@@ -105,11 +111,62 @@ struct litest_test_device* devices[] = {
 	&litest_qemu_tablet_device,
 	&litest_xen_virtual_pointer_device,
 	&litest_vmware_virtmouse_device,
+	&litest_synaptics_hover_device,
+	&litest_synaptics_carbon3rd_device,
 	NULL,
 };
 
 static struct list all_tests;
 
+static void
+litest_reload_udev_rules(void)
+{
+	system("udevadm control --reload-rules");
+}
+
+static int
+litest_udev_rule_filter(const struct dirent *entry)
+{
+	return strncmp(entry->d_name,
+		       UDEV_RULE_PREFIX,
+		       strlen(UDEV_RULE_PREFIX)) == 0;
+}
+
+static void
+litest_drop_udev_rules(void)
+{
+	int n;
+	int rc;
+	struct dirent **entries;
+	char path[PATH_MAX];
+
+	n = scandir(UDEV_RULES_D,
+		    &entries,
+		    litest_udev_rule_filter,
+		    alphasort);
+	if (n < 0)
+		return;
+
+	while (n--) {
+		rc = snprintf(path, sizeof(path),
+			      "%s/%s",
+			      UDEV_RULES_D,
+			      entries[n]->d_name);
+		if (rc > 0 &&
+		    (size_t)rc == strlen(UDEV_RULES_D) +
+			    strlen(entries[n]->d_name) + 1)
+			unlink(path);
+		else
+			fprintf(stderr,
+				"Failed to delete %s. Remaining tests are unreliable\n",
+				entries[n]->d_name);
+		free(entries[n]);
+	}
+	free(entries);
+
+	litest_reload_udev_rules();
+}
+
 static void
 litest_add_tcase_for_device(struct suite *suite,
 			    void *func,
@@ -130,6 +187,13 @@ litest_add_tcase_for_device(struct suite *suite,
 	t->name = strdup(test_name);
 	t->tc = tcase_create(test_name);
 	list_insert(&suite->tests, &t->node);
+	/* we can't guarantee that we clean up properly if a test fails, the
+	   udev rules used for a previous test may still be in place. Add an
+	   unchecked fixture to always clean up all rules before/after a
+	   test case completes */
+	tcase_add_unchecked_fixture(t->tc,
+				    litest_drop_udev_rules,
+				    litest_drop_udev_rules);
 	tcase_add_checked_fixture(t->tc, dev->setup,
 				  dev->teardown ? dev->teardown : litest_generic_device_teardown);
 	tcase_add_test(t->tc, func);
@@ -460,6 +524,40 @@ merge_events(const int *orig, const int *override)
 	return events;
 }
 
+static char *
+litest_init_udev_rules(struct litest_test_device *dev)
+{
+	int rc;
+	FILE *f;
+	char *path;
+
+	if (!dev->udev_rule)
+		return NULL;
+
+	rc = mkdir(UDEV_RULES_D, 0755);
+	if (rc == -1 && errno != EEXIST)
+		ck_abort_msg("Failed to create udev rules directory (%s)\n",
+			     strerror(errno));
+
+	rc = asprintf(&path,
+		      "%s/%s%s.rules",
+		      UDEV_RULES_D,
+		      UDEV_RULE_PREFIX,
+		      dev->shortname);
+	ck_assert_int_eq(rc,
+			 strlen(UDEV_RULES_D) +
+			 strlen(UDEV_RULE_PREFIX) +
+			 strlen(dev->shortname) + 7);
+	f = fopen(path, "w");
+	ck_assert_notnull(f);
+	ck_assert_int_ge(fputs(dev->udev_rule, f), 0);
+	fclose(f);
+
+	litest_reload_udev_rules();
+
+	return path;
+}
+
 static struct litest_device *
 litest_create(enum litest_device_type which,
 	      const char *name_override,
@@ -473,6 +571,7 @@ litest_create(enum litest_device_type which,
 	const struct input_id *id;
 	struct input_absinfo *abs;
 	int *events;
+	char *udev_file;
 
 	dev = devices;
 	while (*dev) {
@@ -487,12 +586,17 @@ litest_create(enum litest_device_type which,
 	d = zalloc(sizeof(*d));
 	ck_assert(d != NULL);
 
+	udev_file = litest_init_udev_rules(*dev);
+
 	/* device has custom create method */
 	if ((*dev)->create) {
 		(*dev)->create(d);
-		if (abs_override || events_override)
+		if (abs_override || events_override) {
+			if (udev_file)
+				unlink(udev_file);
 			ck_abort_msg("Custom create cannot"
 				     "be overridden");
+		}
 
 		return d;
 	}
@@ -507,6 +611,7 @@ litest_create(enum litest_device_type which,
 								 abs,
 								 events);
 	d->interface = (*dev)->interface;
+	d->udev_rule_file = udev_file;
 	free(abs);
 	free(events);
 
@@ -625,6 +730,12 @@ litest_delete_device(struct litest_device *d)
 	if (!d)
 		return;
 
+	if (d->udev_rule_file) {
+		unlink(d->udev_rule_file);
+		free(d->udev_rule_file);
+		d->udev_rule_file = NULL;
+	}
+
 	libinput_device_unref(d->libinput_device);
 	libinput_path_remove_device(d->libinput_device);
 	if (d->owns_context)
@@ -1046,6 +1157,7 @@ litest_create_uinput_device_from_description(const char *name,
 		libevdev_set_id_bustype(dev, id->bustype);
 		libevdev_set_id_vendor(dev, id->vendor);
 		libevdev_set_id_product(dev, id->product);
+		libevdev_set_id_version(dev, id->version);
 	}
 
 	abs = abs_info;
@@ -1277,3 +1389,144 @@ litest_pop_event_frame(struct litest_device *dev)
 	dev->skip_ev_syn = false;
 	litest_event(dev, EV_SYN, SYN_REPORT, 0);
 }
+
+static void
+send_abs_xy(struct litest_device *d, double x, double y)
+{
+	struct input_event e;
+	int val;
+
+	e.type = EV_ABS;
+	e.code = ABS_X;
+	e.value = LITEST_AUTO_ASSIGN;
+	val = litest_auto_assign_value(d, &e, 0, x, y);
+	litest_event(d, EV_ABS, ABS_X, val);
+
+	e.code = ABS_Y;
+	val = litest_auto_assign_value(d, &e, 0, x, y);
+	litest_event(d, EV_ABS, ABS_Y, val);
+}
+
+static void
+send_abs_mt_xy(struct litest_device *d, double x, double y)
+{
+	struct input_event e;
+	int val;
+
+	e.type = EV_ABS;
+	e.code = ABS_MT_POSITION_X;
+	e.value = LITEST_AUTO_ASSIGN;
+	val = litest_auto_assign_value(d, &e, 0, x, y);
+	litest_event(d, EV_ABS, ABS_MT_POSITION_X, val);
+
+	e.code = ABS_MT_POSITION_Y;
+	e.value = LITEST_AUTO_ASSIGN;
+	val = litest_auto_assign_value(d, &e, 0, x, y);
+	litest_event(d, EV_ABS, ABS_MT_POSITION_Y, val);
+}
+
+void
+litest_semi_mt_touch_down(struct litest_device *d,
+			  struct litest_semi_mt *semi_mt,
+			  unsigned int slot,
+			  double x, double y)
+{
+	double t, l, r = 0, b = 0; /* top, left, right, bottom */
+
+	if (d->ntouches_down > 2 || slot > 1)
+		return;
+
+	if (d->ntouches_down == 1) {
+		l = x;
+		t = y;
+	} else {
+		int other = (slot + 1) % 2;
+		l = min(x, semi_mt->touches[other].x);
+		t = min(y, semi_mt->touches[other].y);
+		r = max(x, semi_mt->touches[other].x);
+		b = max(y, semi_mt->touches[other].y);
+	}
+
+	send_abs_xy(d, l, t);
+
+	litest_event(d, EV_ABS, ABS_MT_SLOT, 0);
+
+	if (d->ntouches_down == 1)
+		litest_event(d, EV_ABS, ABS_MT_TRACKING_ID, ++semi_mt->tracking_id);
+
+	send_abs_mt_xy(d, l, t);
+
+	if (d->ntouches_down == 2) {
+		litest_event(d, EV_ABS, ABS_MT_SLOT, 1);
+		litest_event(d, EV_ABS, ABS_MT_TRACKING_ID, ++semi_mt->tracking_id);
+
+		send_abs_mt_xy(d, r, b);
+	}
+
+	litest_event(d, EV_SYN, SYN_REPORT, 0);
+
+	semi_mt->touches[slot].x = x;
+	semi_mt->touches[slot].y = y;
+}
+
+void
+litest_semi_mt_touch_move(struct litest_device *d,
+			  struct litest_semi_mt *semi_mt,
+			  unsigned int slot,
+			  double x, double y)
+{
+	double t, l, r = 0, b = 0; /* top, left, right, bottom */
+
+	if (d->ntouches_down > 2 || slot > 1)
+		return;
+
+	if (d->ntouches_down == 1) {
+		l = x;
+		t = y;
+	} else {
+		int other = (slot + 1) % 2;
+		l = min(x, semi_mt->touches[other].x);
+		t = min(y, semi_mt->touches[other].y);
+		r = max(x, semi_mt->touches[other].x);
+		b = max(y, semi_mt->touches[other].y);
+	}
+
+	send_abs_xy(d, l, t);
+
+	litest_event(d, EV_ABS, ABS_MT_SLOT, 0);
+	send_abs_mt_xy(d, l, t);
+
+	if (d->ntouches_down == 2) {
+		litest_event(d, EV_ABS, ABS_MT_SLOT, 1);
+		send_abs_mt_xy(d, r, b);
+	}
+
+	litest_event(d, EV_SYN, SYN_REPORT, 0);
+
+	semi_mt->touches[slot].x = x;
+	semi_mt->touches[slot].y = y;
+}
+
+void
+litest_semi_mt_touch_up(struct litest_device *d,
+			struct litest_semi_mt *semi_mt,
+			unsigned int slot)
+{
+	/* note: ntouches_down is decreased before we get here */
+	if (d->ntouches_down >= 2 || slot > 1)
+		return;
+
+	litest_event(d, EV_ABS, ABS_MT_SLOT, d->ntouches_down);
+	litest_event(d, EV_ABS, ABS_MT_TRACKING_ID, -1);
+
+	/* if we have one finger left, send x/y coords for that finger left.
+	   this is likely to happen with a real touchpad */
+	if (d->ntouches_down == 1) {
+		int other = (slot + 1) % 2;
+		send_abs_xy(d, semi_mt->touches[other].x, semi_mt->touches[other].y);
+		litest_event(d, EV_ABS, ABS_MT_SLOT, 0);
+		send_abs_mt_xy(d, semi_mt->touches[other].x, semi_mt->touches[other].y);
+	}
+
+	litest_event(d, EV_SYN, SYN_REPORT, 0);
+}
